@@ -10,7 +10,7 @@ use ignore::gitignore::GitignoreBuilder;
 
 use core::{Package, PackageId, Summary, SourceId, Source, Dependency, Registry};
 use ops;
-use util::{self, CargoError, CargoResult, internal};
+use util::{self, CargoResult, internal};
 use util::Config;
 
 pub struct PathSource<'cfg> {
@@ -113,10 +113,10 @@ impl<'cfg> PathSource<'cfg> {
             let pattern: &str = if p.starts_with('/') {
                 &p[1..p.len()]
             } else {
-                &p
+                p
             };
             Pattern::new(pattern).map_err(|e| {
-                CargoError::from(format!("could not parse glob pattern `{}`: {}", p, e))
+                format_err!("could not parse glob pattern `{}`: {}", p, e)
             })
         };
 
@@ -168,10 +168,10 @@ impl<'cfg> PathSource<'cfg> {
                 ) {
                     Match::None => Ok(true),
                     Match::Ignore(_) => Ok(false),
-                    Match::Whitelist(pattern) => Err(CargoError::from(format!(
+                    Match::Whitelist(pattern) => Err(format_err!(
                         "exclude rules cannot start with `!`: {}",
                         pattern.original()
-                    ))),
+                    )),
                 }
             } else {
                 match ignore_include.matched_path_or_any_parents(
@@ -180,10 +180,10 @@ impl<'cfg> PathSource<'cfg> {
                 ) {
                     Match::None => Ok(false),
                     Match::Ignore(_) => Ok(true),
-                    Match::Whitelist(pattern) => Err(CargoError::from(format!(
+                    Match::Whitelist(pattern) => Err(format_err!(
                         "include rules cannot start with `!`: {}",
                         pattern.original()
-                    ))),
+                    )),
                 }
             }
         };
@@ -216,26 +216,24 @@ impl<'cfg> PathSource<'cfg> {
                                 relative_path.display()
                             ))?;
                     }
+                } else if no_include_option {
+                    self.config
+                        .shell()
+                        .warn(format!(
+                            "Pattern matching for Cargo's include/exclude fields is changing and \
+                            file `{}` WILL NOT be excluded in a future Cargo version.\n\
+                            See https://github.com/rust-lang/cargo/issues/4268 for more info",
+                            relative_path.display()
+                        ))?;
                 } else {
-                    if no_include_option {
-                        self.config
-                            .shell()
-                            .warn(format!(
-                                "Pattern matching for Cargo's include/exclude fields is changing and \
-                                file `{}` WILL NOT be excluded in a future Cargo version.\n\
-                                See https://github.com/rust-lang/cargo/issues/4268 for more info",
-                                relative_path.display()
-                            ))?;
-                    } else {
-                        self.config
-                            .shell()
-                            .warn(format!(
-                                "Pattern matching for Cargo's include/exclude fields is changing and \
-                                file `{}` WILL be included in a future Cargo version.\n\
-                                See https://github.com/rust-lang/cargo/issues/4268 for more info",
-                                relative_path.display()
-                            ))?;
-                    }
+                    self.config
+                        .shell()
+                        .warn(format!(
+                            "Pattern matching for Cargo's include/exclude fields is changing and \
+                            file `{}` WILL be included in a future Cargo version.\n\
+                            See https://github.com/rust-lang/cargo/issues/4268 for more info",
+                            relative_path.display()
+                        ))?;
                 }
             }
 
@@ -294,7 +292,7 @@ impl<'cfg> PathSource<'cfg> {
                 None => break,
             }
         }
-        return None;
+        None
     }
 
     fn list_files_git(&self, pkg: &Package, repo: git2::Repository,
@@ -379,7 +377,7 @@ impl<'cfg> PathSource<'cfg> {
                 warn!("  found submodule {}", file_path.display());
                 let rel = util::without_prefix(&file_path, root).unwrap();
                 let rel = rel.to_str().ok_or_else(|| {
-                    CargoError::from(format!("invalid utf-8 filename: {}", rel.display()))
+                    format_err!("invalid utf-8 filename: {}", rel.display())
                 })?;
                 // Git submodules are currently only named through `/` path
                 // separators, explicitly not `\` which windows uses. Who knew?
@@ -484,6 +482,14 @@ impl<'cfg> Registry for PathSource<'cfg> {
         }
         Ok(())
     }
+
+    fn supports_checksums(&self) -> bool {
+        false
+    }
+
+    fn requires_precise(&self) -> bool {
+        false
+    }
 }
 
 impl<'cfg> Source for PathSource<'cfg> {
