@@ -16,8 +16,9 @@ fn cargo_bench_simple() {
 
     let p = project("foo")
         .file("Cargo.toml", &basic_bin_manifest("foo"))
-        .file("src/foo.rs", r#"
+        .file("src/main.rs", r#"
             #![feature(test)]
+            #[cfg(test)]
             extern crate test;
 
             fn hello() -> &'static str {
@@ -31,9 +32,10 @@ fn cargo_bench_simple() {
             #[bench]
             fn bench_hello(_b: &mut test::Bencher) {
                 assert_eq!(hello(), "hello")
-            }"#);
+            }"#)
+        .build();
 
-    assert_that(p.cargo_process("build"), execs());
+    assert_that(p.cargo("build"), execs());
     assert_that(&p.bin("foo"), existing_file());
 
     assert_that(process(&p.bin("foo")),
@@ -44,20 +46,89 @@ fn cargo_bench_simple() {
 [COMPILING] foo v0.5.0 ({})
 [FINISHED] release [optimized] target(s) in [..]
 [RUNNING] target[/]release[/]deps[/]foo-[..][EXE]", p.url()))
-                       .with_stdout("
-running 1 test
-test bench_hello ... bench: [..] 0 ns/iter (+/- 0)
+                       .with_stdout_contains("test bench_hello ... bench: [..]"));
+}
 
-test result: ok. 0 passed; 0 failed; 0 ignored; 1 measured
+#[test]
+fn bench_bench_implicit() {
+    if !is_nightly() { return }
 
-"));
+    let p = project("foo")
+        .file("Cargo.toml" , r#"
+            [package]
+            name = "foo"
+            version = "0.0.1"
+            authors = []
+        "#)
+        .file("src/main.rs", r#"
+            #![cfg_attr(test, feature(test))]
+            #[cfg(test)]
+            extern crate test;
+            #[bench] fn run1(_ben: &mut test::Bencher) { }
+            fn main() { println!("Hello main!"); }"#)
+        .file("tests/other.rs", r#"
+            #![feature(test)]
+            extern crate test;
+            #[bench] fn run3(_ben: &mut test::Bencher) { }"#)
+        .file("benches/mybench.rs", r#"
+            #![feature(test)]
+            extern crate test;
+            #[bench] fn run2(_ben: &mut test::Bencher) { }"#)
+        .build();
+
+    assert_that(p.cargo("bench").arg("--benches"),
+        execs().with_status(0)
+               .with_stderr(format!("\
+[COMPILING] foo v0.0.1 ({dir})
+[FINISHED] release [optimized] target(s) in [..]
+[RUNNING] target[/]release[/]deps[/]foo-[..][EXE]
+[RUNNING] target[/]release[/]deps[/]mybench-[..][EXE]
+", dir = p.url()))
+               .with_stdout_contains("test run2 ... bench: [..]"));
+}
+
+#[test]
+fn bench_bin_implicit() {
+    if !is_nightly() { return }
+
+    let p = project("foo")
+        .file("Cargo.toml" , r#"
+            [package]
+            name = "foo"
+            version = "0.0.1"
+            authors = []
+        "#)
+        .file("src/main.rs", r#"
+            #![feature(test)]
+            #[cfg(test)]
+            extern crate test;
+            #[bench] fn run1(_ben: &mut test::Bencher) { }
+            fn main() { println!("Hello main!"); }"#)
+        .file("tests/other.rs", r#"
+            #![feature(test)]
+            extern crate test;
+            #[bench] fn run3(_ben: &mut test::Bencher) { }"#)
+        .file("benches/mybench.rs", r#"
+            #![feature(test)]
+            extern crate test;
+            #[bench] fn run2(_ben: &mut test::Bencher) { }"#)
+        .build();
+
+    assert_that(p.cargo("bench").arg("--bins"),
+        execs().with_status(0)
+               .with_stderr(format!("\
+[COMPILING] foo v0.0.1 ({dir})
+[FINISHED] release [optimized] target(s) in [..]
+[RUNNING] target[/]release[/]deps[/]foo-[..][EXE]
+", dir = p.url()))
+               .with_stdout_contains("test run1 ... bench: [..]"));
 }
 
 #[test]
 fn bench_tarname() {
     if !is_nightly() { return }
 
-    let prj = project("foo")
+    let p = project("foo")
         .file("Cargo.toml" , r#"
             [package]
             name = "foo"
@@ -71,22 +142,52 @@ fn bench_tarname() {
         .file("benches/bin2.rs", r#"
             #![feature(test)]
             extern crate test;
-            #[bench] fn run2(_ben: &mut test::Bencher) { }"#);
+            #[bench] fn run2(_ben: &mut test::Bencher) { }"#)
+        .build();
 
-    assert_that(prj.cargo_process("bench").arg("--bench").arg("bin2"),
+    assert_that(p.cargo("bench").arg("--bench").arg("bin2"),
         execs().with_status(0)
                .with_stderr(format!("\
 [COMPILING] foo v0.0.1 ({dir})
 [FINISHED] release [optimized] target(s) in [..]
 [RUNNING] target[/]release[/]deps[/]bin2-[..][EXE]
-", dir = prj.url()))
-               .with_stdout("
-running 1 test
-test run2 ... bench: [..] 0 ns/iter (+/- 0)
+", dir = p.url()))
+               .with_stdout_contains("test run2 ... bench: [..]"));
+}
 
-test result: ok. 0 passed; 0 failed; 0 ignored; 1 measured
+#[test]
+fn bench_multiple_targets() {
+    if !is_nightly() { return }
 
-"));
+    let p = project("foo")
+        .file("Cargo.toml" , r#"
+            [package]
+            name = "foo"
+            version = "0.0.1"
+            authors = []
+        "#)
+        .file("benches/bin1.rs", r#"
+            #![feature(test)]
+            extern crate test;
+            #[bench] fn run1(_ben: &mut test::Bencher) { }"#)
+        .file("benches/bin2.rs", r#"
+            #![feature(test)]
+            extern crate test;
+            #[bench] fn run2(_ben: &mut test::Bencher) { }"#)
+        .file("benches/bin3.rs", r#"
+            #![feature(test)]
+            extern crate test;
+            #[bench] fn run3(_ben: &mut test::Bencher) { }"#)
+        .build();
+
+    assert_that(p.cargo("bench")
+                .arg("--bench").arg("bin1")
+                .arg("--bench").arg("bin2"),
+                execs()
+                .with_status(0)
+                .with_stdout_contains("test run1 ... bench: [..]")
+                .with_stdout_contains("test run2 ... bench: [..]")
+                .with_stdout_does_not_contain("run3"));
 }
 
 #[test]
@@ -95,26 +196,22 @@ fn cargo_bench_verbose() {
 
     let p = project("foo")
         .file("Cargo.toml", &basic_bin_manifest("foo"))
-        .file("src/foo.rs", r#"
+        .file("src/main.rs", r#"
             #![feature(test)]
+            #[cfg(test)]
             extern crate test;
             fn main() {}
             #[bench] fn bench_hello(_b: &mut test::Bencher) {}
-        "#);
+        "#)
+        .build();
 
-    assert_that(p.cargo_process("bench").arg("-v").arg("hello"),
+    assert_that(p.cargo("bench").arg("-v").arg("hello"),
                 execs().with_stderr(&format!("\
 [COMPILING] foo v0.5.0 ({url})
-[RUNNING] `rustc [..] src[/]foo.rs [..]`
+[RUNNING] `rustc [..] src[/]main.rs [..]`
 [FINISHED] release [optimized] target(s) in [..]
 [RUNNING] `[..]target[/]release[/]deps[/]foo-[..][EXE] hello --bench`", url = p.url()))
-                       .with_stdout("
-running 1 test
-test bench_hello ... bench: [..] 0 ns/iter (+/- 0)
-
-test result: ok. 0 passed; 0 failed; 0 ignored; 1 measured
-
-"));
+                       .with_stdout_contains("test bench_hello ... bench: [..]"));
 }
 
 #[test]
@@ -130,13 +227,16 @@ fn many_similar_names() {
         "#)
         .file("src/lib.rs", "
             #![feature(test)]
+            #[cfg(test)]
             extern crate test;
             pub fn foo() {}
             #[bench] fn lib_bench(_b: &mut test::Bencher) {}
         ")
         .file("src/main.rs", "
             #![feature(test)]
+            #[cfg(test)]
             extern crate foo;
+            #[cfg(test)]
             extern crate test;
             fn main() {}
             #[bench] fn bin_bench(_b: &mut test::Bencher) { foo::foo() }
@@ -146,9 +246,10 @@ fn many_similar_names() {
             extern crate foo;
             extern crate test;
             #[bench] fn bench_bench(_b: &mut test::Bencher) { foo::foo() }
-        "#);
+        "#)
+        .build();
 
-    let output = p.cargo_process("bench").exec_with_output().unwrap();
+    let output = p.cargo("bench").exec_with_output().unwrap();
     let output = str::from_utf8(&output.stdout).unwrap();
     assert!(output.contains("test bin_bench"), "bin_bench missing\n{}", output);
     assert!(output.contains("test lib_bench"), "lib_bench missing\n{}", output);
@@ -156,13 +257,15 @@ fn many_similar_names() {
 }
 
 #[test]
+#[ignore] // unignored on the master branch of cargo
 fn cargo_bench_failing_test() {
     if !is_nightly() { return }
 
     let p = project("foo")
         .file("Cargo.toml", &basic_bin_manifest("foo"))
-        .file("src/foo.rs", r#"
+        .file("src/main.rs", r#"
             #![feature(test)]
+            #[cfg(test)]
             extern crate test;
             fn hello() -> &'static str {
                 "hello"
@@ -175,27 +278,26 @@ fn cargo_bench_failing_test() {
             #[bench]
             fn bench_hello(_b: &mut test::Bencher) {
                 assert_eq!(hello(), "nope")
-            }"#);
+            }"#)
+        .build();
 
-    assert_that(p.cargo_process("build"), execs());
+    assert_that(p.cargo("build"), execs());
     assert_that(&p.bin("foo"), existing_file());
 
     assert_that(process(&p.bin("foo")),
                 execs().with_stdout("hello\n"));
 
     assert_that(p.cargo("bench"),
-                execs().with_stdout_contains("
-running 1 test
-test bench_hello ... ")
+                execs().with_stdout_contains("test bench_hello ... ")
                        .with_stderr_contains(format!("\
 [COMPILING] foo v0.5.0 ({})
 [FINISHED] release [optimized] target(s) in [..]
 [RUNNING] target[/]release[/]deps[/]foo-[..][EXE]
 thread '[..]' panicked at 'assertion failed: \
-    `(left == right)` (left: \
-    `\"hello\"`, right: `\"nope\"`)', src[/]foo.rs:14
-[..]
-", p.url()))
+    `(left == right)`[..]", p.url()))
+                       .with_stderr_contains("[..]left: `\"hello\"`[..]")
+                       .with_stderr_contains("[..]right: `\"nope\"`[..]")
+                       .with_stderr_contains("[..]src[/]main.rs:15[..]")
                        .with_status(101));
 }
 
@@ -215,7 +317,8 @@ fn bench_with_lib_dep() {
             path = "src/main.rs"
         "#)
         .file("src/lib.rs", r#"
-            #![feature(test)]
+            #![cfg_attr(test, feature(test))]
+            #[cfg(test)]
             extern crate test;
             ///
             /// ```rust
@@ -230,34 +333,26 @@ fn bench_with_lib_dep() {
         "#)
         .file("src/main.rs", "
             #![feature(test)]
+            #[allow(unused_extern_crates)]
             extern crate foo;
+            #[cfg(test)]
             extern crate test;
 
             fn main() {}
 
             #[bench]
             fn bin_bench(_b: &mut test::Bencher) {}
-        ");
+        ")
+        .build();
 
-    assert_that(p.cargo_process("bench"),
+    assert_that(p.cargo("bench"),
                 execs().with_stderr(&format!("\
 [COMPILING] foo v0.0.1 ({})
 [FINISHED] release [optimized] target(s) in [..]
 [RUNNING] target[/]release[/]deps[/]foo-[..][EXE]
 [RUNNING] target[/]release[/]deps[/]baz-[..][EXE]", p.url()))
-                       .with_stdout("
-running 1 test
-test lib_bench ... bench: [..] 0 ns/iter (+/- 0)
-
-test result: ok. 0 passed; 0 failed; 0 ignored; 1 measured
-
-
-running 1 test
-test bin_bench ... bench: [..] 0 ns/iter (+/- 0)
-
-test result: ok. 0 passed; 0 failed; 0 ignored; 1 measured
-
-"))
+                       .with_stdout_contains("test lib_bench ... bench: [..]")
+                       .with_stdout_contains("test bin_bench ... bench: [..]"));
 }
 
 #[test]
@@ -275,15 +370,18 @@ fn bench_with_deep_lib_dep() {
             path = "../foo"
         "#)
         .file("src/lib.rs", "
-            #![feature(test)]
+            #![cfg_attr(test, feature(test))]
+            #[cfg(test)]
             extern crate foo;
+            #[cfg(test)]
             extern crate test;
             #[bench]
             fn bar_bench(_b: &mut test::Bencher) {
                 foo::foo();
             }
-        ");
-    let p2 = project("foo")
+        ")
+        .build();
+    let _p2 = project("foo")
         .file("Cargo.toml", r#"
             [package]
             name = "foo"
@@ -291,30 +389,25 @@ fn bench_with_deep_lib_dep() {
             authors = []
         "#)
         .file("src/lib.rs", "
-            #![feature(test)]
+            #![cfg_attr(test, feature(test))]
+            #[cfg(test)]
             extern crate test;
 
             pub fn foo() {}
 
             #[bench]
             fn foo_bench(_b: &mut test::Bencher) {}
-        ");
+        ")
+        .build();
 
-    p2.build();
-    assert_that(p.cargo_process("bench"),
+    assert_that(p.cargo("bench"),
                 execs().with_status(0)
                        .with_stderr(&format!("\
 [COMPILING] foo v0.0.1 ([..])
 [COMPILING] bar v0.0.1 ({dir})
 [FINISHED] release [optimized] target(s) in [..]
 [RUNNING] target[/]release[/]deps[/]bar-[..][EXE]", dir = p.url()))
-                       .with_stdout("
-running 1 test
-test bar_bench ... bench: [..] 0 ns/iter (+/- 0)
-
-test result: ok. 0 passed; 0 failed; 0 ignored; 1 measured
-
-"));
+                       .with_stdout_contains("test bar_bench ... bench: [..]"));
 }
 
 #[test]
@@ -333,7 +426,8 @@ fn external_bench_explicit() {
             path = "src/bench.rs"
         "#)
         .file("src/lib.rs", r#"
-            #![feature(test)]
+            #![cfg_attr(test, feature(test))]
+            #[cfg(test)]
             extern crate test;
             pub fn get_hello() -> &'static str { "Hello" }
 
@@ -342,32 +436,23 @@ fn external_bench_explicit() {
         "#)
         .file("src/bench.rs", r#"
             #![feature(test)]
+            #[allow(unused_extern_crates)]
             extern crate foo;
             extern crate test;
 
             #[bench]
             fn external_bench(_b: &mut test::Bencher) {}
-        "#);
+        "#)
+        .build();
 
-    assert_that(p.cargo_process("bench"),
+    assert_that(p.cargo("bench"),
                 execs().with_stderr(&format!("\
 [COMPILING] foo v0.0.1 ({})
 [FINISHED] release [optimized] target(s) in [..]
 [RUNNING] target[/]release[/]deps[/]foo-[..][EXE]
 [RUNNING] target[/]release[/]deps[/]bench-[..][EXE]", p.url()))
-                       .with_stdout("
-running 1 test
-test internal_bench ... bench: [..] 0 ns/iter (+/- 0)
-
-test result: ok. 0 passed; 0 failed; 0 ignored; 1 measured
-
-
-running 1 test
-test external_bench ... bench: [..] 0 ns/iter (+/- 0)
-
-test result: ok. 0 passed; 0 failed; 0 ignored; 1 measured
-
-"))
+                       .with_stdout_contains("test internal_bench ... bench: [..]")
+                       .with_stdout_contains("test external_bench ... bench: [..]"));
 }
 
 #[test]
@@ -382,7 +467,8 @@ fn external_bench_implicit() {
             authors = []
         "#)
         .file("src/lib.rs", r#"
-            #![feature(test)]
+            #![cfg_attr(test, feature(test))]
+            #[cfg(test)]
             extern crate test;
 
             pub fn get_hello() -> &'static str { "Hello" }
@@ -392,32 +478,23 @@ fn external_bench_implicit() {
         "#)
         .file("benches/external.rs", r#"
             #![feature(test)]
+            #[allow(unused_extern_crates)]
             extern crate foo;
             extern crate test;
 
             #[bench]
             fn external_bench(_b: &mut test::Bencher) {}
-        "#);
+        "#)
+        .build();
 
-    assert_that(p.cargo_process("bench"),
+    assert_that(p.cargo("bench"),
                 execs().with_stderr(&format!("\
 [COMPILING] foo v0.0.1 ({})
 [FINISHED] release [optimized] target(s) in [..]
 [RUNNING] target[/]release[/]deps[/]foo-[..][EXE]
 [RUNNING] target[/]release[/]deps[/]external-[..][EXE]", p.url()))
-                       .with_stdout("
-running 1 test
-test internal_bench ... bench: [..] 0 ns/iter (+/- 0)
-
-test result: ok. 0 passed; 0 failed; 0 ignored; 1 measured
-
-
-running 1 test
-test external_bench ... bench: [..] 0 ns/iter (+/- 0)
-
-test result: ok. 0 passed; 0 failed; 0 ignored; 1 measured
-
-"))
+                       .with_stdout_contains("test internal_bench ... bench: [..]")
+                       .with_stdout_contains("test external_bench ... bench: [..]"));
 }
 
 #[test]
@@ -431,12 +508,12 @@ fn dont_run_examples() {
             version = "0.0.1"
             authors = []
         "#)
-        .file("src/lib.rs", r#"
-        "#)
+        .file("src/lib.rs", r"")
         .file("examples/dont-run-me-i-will-fail.rs", r#"
             fn main() { panic!("Examples should not be run by 'cargo test'"); }
-        "#);
-    assert_that(p.cargo_process("bench"),
+        "#)
+        .build();
+    assert_that(p.cargo("bench"),
                 execs().with_status(0));
 }
 
@@ -453,37 +530,27 @@ fn pass_through_command_line() {
         "#)
         .file("src/lib.rs", "
             #![feature(test)]
+            #[cfg(test)]
             extern crate test;
 
             #[bench] fn foo(_b: &mut test::Bencher) {}
             #[bench] fn bar(_b: &mut test::Bencher) {}
-        ");
+        ")
+        .build();
 
-    assert_that(p.cargo_process("bench").arg("bar"),
+    assert_that(p.cargo("bench").arg("bar"),
                 execs().with_status(0)
                 .with_stderr(&format!("\
 [COMPILING] foo v0.0.1 ({dir})
 [FINISHED] release [optimized] target(s) in [..]
 [RUNNING] target[/]release[/]deps[/]foo-[..][EXE]", dir = p.url()))
-                .with_stdout("
-running 1 test
-test bar ... bench: [..] 0 ns/iter (+/- 0)
-
-test result: ok. 0 passed; 0 failed; 0 ignored; 1 measured
-
-"));
+                .with_stdout_contains("test bar ... bench: [..]"));
 
     assert_that(p.cargo("bench").arg("foo"),
                 execs().with_status(0)
                        .with_stderr("[FINISHED] release [optimized] target(s) in [..]
 [RUNNING] target[/]release[/]deps[/]foo-[..][EXE]")
-                       .with_stdout("
-running 1 test
-test foo ... bench: [..] 0 ns/iter (+/- 0)
-
-test result: ok. 0 passed; 0 failed; 0 ignored; 1 measured
-
-"));
+                       .with_stdout_contains("test foo ... bench: [..]"));
 }
 
 // Regression test for running cargo-bench twice with
@@ -497,14 +564,15 @@ fn cargo_bench_twice() {
         .file("src/test_twice.rs", r#"
             #![crate_type = "rlib"]
             #![feature(test)]
-
+            #[cfg(test)]
             extern crate test;
 
             #[bench]
             fn dummy_bench(b: &mut test::Bencher) { }
-            "#);
+            "#)
+        .build();
 
-    p.cargo_process("build");
+    p.cargo("build");
 
     for _ in 0..2 {
         assert_that(p.cargo("bench"),
@@ -529,38 +597,30 @@ fn lib_bin_same_name() {
             name = "foo"
         "#)
         .file("src/lib.rs", "
-            #![feature(test)]
+            #![cfg_attr(test, feature(test))]
+            #[cfg(test)]
             extern crate test;
             #[bench] fn lib_bench(_b: &mut test::Bencher) {}
         ")
         .file("src/main.rs", "
-            #![feature(test)]
+            #![cfg_attr(test, feature(test))]
+            #[allow(unused_extern_crates)]
             extern crate foo;
+            #[cfg(test)]
             extern crate test;
 
             #[bench]
             fn bin_bench(_b: &mut test::Bencher) {}
-        ");
+        ")
+        .build();
 
-    assert_that(p.cargo_process("bench"),
+    assert_that(p.cargo("bench"),
                 execs().with_stderr(&format!("\
 [COMPILING] foo v0.0.1 ({})
 [FINISHED] release [optimized] target(s) in [..]
 [RUNNING] target[/]release[/]deps[/]foo-[..][EXE]
 [RUNNING] target[/]release[/]deps[/]foo-[..][EXE]", p.url()))
-                       .with_stdout("
-running 1 test
-test [..] ... bench: [..] 0 ns/iter (+/- 0)
-
-test result: ok. 0 passed; 0 failed; 0 ignored; 1 measured
-
-
-running 1 test
-test [..] ... bench: [..] 0 ns/iter (+/- 0)
-
-test result: ok. 0 passed; 0 failed; 0 ignored; 1 measured
-
-"))
+                       .with_stdout_contains_n("test [..] ... bench: [..]", 2));
 }
 
 #[test]
@@ -575,7 +635,8 @@ fn lib_with_standard_name() {
             authors = []
         "#)
         .file("src/lib.rs", "
-            #![feature(test)]
+            #![cfg_attr(test, feature(test))]
+            #[cfg(test)]
             extern crate test;
 
             /// ```
@@ -593,28 +654,18 @@ fn lib_with_standard_name() {
 
             #[bench]
             fn bench(_b: &mut test::Bencher) { syntax::foo() }
-        ");
+        ")
+        .build();
 
-    assert_that(p.cargo_process("bench"),
+    assert_that(p.cargo("bench"),
                 execs().with_status(0)
                        .with_stderr(&format!("\
 [COMPILING] syntax v0.0.1 ({dir})
 [FINISHED] release [optimized] target(s) in [..]
 [RUNNING] target[/]release[/]deps[/]syntax-[..][EXE]
 [RUNNING] target[/]release[/]deps[/]bench-[..][EXE]", dir = p.url()))
-                       .with_stdout("
-running 1 test
-test foo_bench ... bench: [..] 0 ns/iter (+/- 0)
-
-test result: ok. 0 passed; 0 failed; 0 ignored; 1 measured
-
-
-running 1 test
-test bench ... bench: [..] 0 ns/iter (+/- 0)
-
-test result: ok. 0 passed; 0 failed; 0 ignored; 1 measured
-
-"));
+                       .with_stdout_contains("test foo_bench ... bench: [..]")
+                       .with_stdout_contains("test bench ... bench: [..]"));
 }
 
 #[test]
@@ -638,28 +689,25 @@ fn lib_with_standard_name2() {
         ")
         .file("src/main.rs", "
             #![feature(test)]
+            #[cfg(test)]
             extern crate syntax;
+            #[cfg(test)]
             extern crate test;
 
             fn main() {}
 
             #[bench]
             fn bench(_b: &mut test::Bencher) { syntax::foo() }
-        ");
+        ")
+        .build();
 
-    assert_that(p.cargo_process("bench"),
+    assert_that(p.cargo("bench"),
                 execs().with_status(0)
                        .with_stderr(&format!("\
 [COMPILING] syntax v0.0.1 ({dir})
 [FINISHED] release [optimized] target(s) in [..]
 [RUNNING] target[/]release[/]deps[/]syntax-[..][EXE]", dir = p.url()))
-                       .with_stdout("
-running 1 test
-test bench ... bench: [..] 0 ns/iter (+/- 0)
-
-test result: ok. 0 passed; 0 failed; 0 ignored; 1 measured
-
-"));
+                       .with_stdout_contains("test bench ... bench: [..]"));
 }
 
 #[test]
@@ -681,8 +729,9 @@ fn bench_dylib() {
             path = "bar"
         "#)
         .file("src/lib.rs", r#"
-            #![feature(test)]
+            #![cfg_attr(test, feature(test))]
             extern crate bar as the_bar;
+            #[cfg(test)]
             extern crate test;
 
             pub fn bar() { the_bar::baz(); }
@@ -710,9 +759,10 @@ fn bench_dylib() {
         "#)
         .file("bar/src/lib.rs", "
              pub fn baz() {}
-        ");
+        ")
+        .build();
 
-    assert_that(p.cargo_process("bench").arg("-v"),
+    assert_that(p.cargo("bench").arg("-v"),
                 execs().with_status(0)
                        .with_stderr(&format!("\
 [COMPILING] bar v0.0.1 ({dir}/bar)
@@ -724,19 +774,8 @@ fn bench_dylib() {
 [FINISHED] release [optimized] target(s) in [..]
 [RUNNING] `[..]target[/]release[/]deps[/]foo-[..][EXE] --bench`
 [RUNNING] `[..]target[/]release[/]deps[/]bench-[..][EXE] --bench`", dir = p.url()))
-                       .with_stdout("
-running 1 test
-test foo ... bench: [..] 0 ns/iter (+/- 0)
+                       .with_stdout_contains_n("test foo ... bench: [..]", 2));
 
-test result: ok. 0 passed; 0 failed; 0 ignored; 1 measured
-
-
-running 1 test
-test foo ... bench: [..] 0 ns/iter (+/- 0)
-
-test result: ok. 0 passed; 0 failed; 0 ignored; 1 measured
-
-"));
     p.root().move_into_the_past();
     assert_that(p.cargo("bench").arg("-v"),
                 execs().with_status(0)
@@ -746,19 +785,7 @@ test result: ok. 0 passed; 0 failed; 0 ignored; 1 measured
 [FINISHED] release [optimized] target(s) in [..]
 [RUNNING] `[..]target[/]release[/]deps[/]foo-[..][EXE] --bench`
 [RUNNING] `[..]target[/]release[/]deps[/]bench-[..][EXE] --bench`", dir = p.url()))
-                       .with_stdout("
-running 1 test
-test foo ... bench: [..] 0 ns/iter (+/- 0)
-
-test result: ok. 0 passed; 0 failed; 0 ignored; 1 measured
-
-
-running 1 test
-test foo ... bench: [..] 0 ns/iter (+/- 0)
-
-test result: ok. 0 passed; 0 failed; 0 ignored; 1 measured
-
-"));
+                       .with_stdout_contains_n("test foo ... bench: [..]", 2));
 }
 
 #[test]
@@ -776,36 +803,26 @@ fn bench_twice_with_build_cmd() {
         .file("build.rs", "fn main() {}")
         .file("src/lib.rs", "
             #![feature(test)]
+            #[cfg(test)]
             extern crate test;
             #[bench]
             fn foo(_b: &mut test::Bencher) {}
-        ");
+        ")
+        .build();
 
-    assert_that(p.cargo_process("bench"),
+    assert_that(p.cargo("bench"),
                 execs().with_status(0)
                        .with_stderr(&format!("\
 [COMPILING] foo v0.0.1 ({dir})
 [FINISHED] release [optimized] target(s) in [..]
 [RUNNING] target[/]release[/]deps[/]foo-[..][EXE]", dir = p.url()))
-                       .with_stdout("
-running 1 test
-test foo ... bench: [..] 0 ns/iter (+/- 0)
-
-test result: ok. 0 passed; 0 failed; 0 ignored; 1 measured
-
-"));
+                       .with_stdout_contains("test foo ... bench: [..]"));
 
     assert_that(p.cargo("bench"),
                 execs().with_status(0)
                        .with_stderr("[FINISHED] release [optimized] target(s) in [..]
 [RUNNING] target[/]release[/]deps[/]foo-[..][EXE]")
-                       .with_stdout("
-running 1 test
-test foo ... bench: [..] 0 ns/iter (+/- 0)
-
-test result: ok. 0 passed; 0 failed; 0 ignored; 1 measured
-
-"));
+                       .with_stdout_contains("test foo ... bench: [..]"));
 }
 
 #[test]
@@ -826,7 +843,8 @@ fn bench_with_examples() {
             name = "testb1"
         "#)
         .file("src/lib.rs", r#"
-            #![feature(test)]
+            #![cfg_attr(test, feature(test))]
+            #[cfg(test)]
             extern crate test;
             #[cfg(test)]
             use test::Bencher;
@@ -861,9 +879,10 @@ fn bench_with_examples() {
                 println!("example1");
                 testbench::f1();
             }
-        "#);
+        "#)
+        .build();
 
-    assert_that(p.cargo_process("bench").arg("-v"),
+    assert_that(p.cargo("bench").arg("-v"),
                 execs().with_status(0)
                        .with_stderr(&format!("\
 [COMPILING] testbench v6.6.6 ({url})
@@ -874,19 +893,8 @@ fn bench_with_examples() {
 [RUNNING] `{dir}[/]target[/]release[/]deps[/]testbench-[..][EXE] --bench`
 [RUNNING] `{dir}[/]target[/]release[/]deps[/]testb1-[..][EXE] --bench`",
                 dir = p.root().display(), url = p.url()))
-                       .with_stdout("
-running 1 test
-test bench_bench1 ... bench: [..] 0 ns/iter (+/- 0)
-
-test result: ok. 0 passed; 0 failed; 0 ignored; 1 measured
-
-
-running 1 test
-test bench_bench2 ... bench: [..] 0 ns/iter (+/- 0)
-
-test result: ok. 0 passed; 0 failed; 0 ignored; 1 measured
-
-"));
+                       .with_stdout_contains("test bench_bench1 ... bench: [..]")
+                       .with_stdout_contains("test bench_bench2 ... bench: [..]"));
 }
 
 #[test]
@@ -913,21 +921,16 @@ fn test_a_bench() {
         .file("benches/b.rs", r#"
             #[test]
             fn foo() {}
-        "#);
+        "#)
+        .build();
 
-    assert_that(p.cargo_process("test"),
+    assert_that(p.cargo("test"),
                 execs().with_status(0)
                        .with_stderr("\
 [COMPILING] foo v0.1.0 ([..])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 [RUNNING] target[/]debug[/]deps[/]b-[..][EXE]")
-                       .with_stdout("
-running 1 test
-test foo ... ok
-
-test result: ok. 1 passed; 0 failed; 0 ignored; 0 measured
-
-"));
+                       .with_stdout_contains("test foo ... ok"));
 }
 
 #[test]
@@ -951,9 +954,10 @@ fn test_bench_no_run() {
 
             #[bench]
             fn bench_baz(_: &mut Bencher) {}
-        "#);
+        "#)
+        .build();
 
-    assert_that(p.cargo_process("bench").arg("--no-run"),
+    assert_that(p.cargo("bench").arg("--no-run"),
                 execs().with_status(0)
                        .with_stderr("\
 [COMPILING] foo v0.1.0 ([..])
@@ -961,6 +965,46 @@ fn test_bench_no_run() {
 "));
 }
 
+#[test]
+fn test_bench_no_fail_fast() {
+    if !is_nightly() { return }
+
+    let p = project("foo")
+        .file("Cargo.toml", &basic_bin_manifest("foo"))
+        .file("src/foo.rs", r#"
+            #![feature(test)]
+            #[cfg(test)]
+            extern crate test;
+            fn hello() -> &'static str {
+                "hello"
+            }
+
+            pub fn main() {
+                println!("{}", hello())
+            }
+
+            #[bench]
+            fn bench_hello(_b: &mut test::Bencher) {
+                assert_eq!(hello(), "hello")
+            }
+
+            #[bench]
+            fn bench_nope(_b: &mut test::Bencher) {
+                assert_eq!("nope", hello())
+            }"#)
+        .build();
+
+    assert_that(p.cargo("bench").arg("--no-fail-fast"),
+                execs().with_status(101)
+                    .with_stderr_contains("\
+[RUNNING] target[/]release[/]deps[/]foo-[..][EXE]")
+                    .with_stdout_contains("running 2 tests")
+                    .with_stderr_contains("\
+[RUNNING] target[/]release[/]deps[/]foo-[..][EXE]")
+                    .with_stdout_contains("test bench_hello [..]")
+                    .with_stdout_contains("test bench_nope [..]"));
+}
+
 #[test]
 fn test_bench_multiple_packages() {
     if !is_nightly() { return }
@@ -978,9 +1022,10 @@ fn test_bench_multiple_packages() {
             [dependencies.baz]
             path = "../baz"
         "#)
-        .file("src/lib.rs", "");
+        .file("src/lib.rs", "")
+        .build();
 
-    let bar = project("bar")
+    let _bar = project("bar")
         .file("Cargo.toml", r#"
             [project]
             name = "bar"
@@ -1000,10 +1045,10 @@ fn test_bench_multiple_packages() {
 
             #[bench]
             fn bench_bar(_b: &mut Bencher) {}
-        "#);
-    bar.build();
+        "#)
+        .build();
 
-    let baz = project("baz")
+    let _baz = project("baz")
         .file("Cargo.toml", r#"
             [project]
             name = "baz"
@@ -1023,26 +1068,270 @@ fn test_bench_multiple_packages() {
 
             #[bench]
             fn bench_baz(_b: &mut Bencher) {}
-        "#);
-    baz.build();
+        "#)
+        .build();
 
 
-    assert_that(p.cargo_process("bench").arg("-p").arg("bar").arg("-p").arg("baz"),
+    assert_that(p.cargo("bench").arg("-p").arg("bar").arg("-p").arg("baz"),
                 execs().with_status(0)
                        .with_stderr_contains("\
 [RUNNING] target[/]release[/]deps[/]bbaz-[..][EXE]")
-                       .with_stdout_contains("
-running 1 test
-test bench_baz ... bench:           0 ns/iter (+/- 0)
-
-test result: ok. 0 passed; 0 failed; 0 ignored; 1 measured
-")
+                       .with_stdout_contains("test bench_baz ... bench: [..]")
                        .with_stderr_contains("\
 [RUNNING] target[/]release[/]deps[/]bbar-[..][EXE]")
-                       .with_stdout_contains("
+                       .with_stdout_contains("test bench_bar ... bench: [..]"));
+}
+
+#[test]
+fn bench_all_workspace() {
+    if !is_nightly() { return }
+
+    let p = project("foo")
+        .file("Cargo.toml", r#"
+            [project]
+            name = "foo"
+            version = "0.1.0"
+
+            [dependencies]
+            bar = { path = "bar" }
+
+            [workspace]
+        "#)
+        .file("src/main.rs", r#"
+            fn main() {}
+        "#)
+        .file("benches/foo.rs", r#"
+            #![feature(test)]
+            extern crate test;
+
+            use test::Bencher;
+
+            #[bench]
+            fn bench_foo(_: &mut Bencher) -> () { () }
+        "#)
+        .file("bar/Cargo.toml", r#"
+            [project]
+            name = "bar"
+            version = "0.1.0"
+        "#)
+        .file("bar/src/lib.rs", r#"
+            pub fn bar() {}
+        "#)
+        .file("bar/benches/bar.rs", r#"
+            #![feature(test)]
+            extern crate test;
+
+            use test::Bencher;
+
+            #[bench]
+            fn bench_bar(_: &mut Bencher) -> () { () }
+        "#)
+        .build();
+
+    assert_that(p.cargo("bench")
+                 .arg("--all"),
+                execs().with_status(0)
+                       .with_stderr_contains("\
+[RUNNING] target[/]release[/]deps[/]bar-[..][EXE]")
+                       .with_stdout_contains("test bench_bar ... bench: [..]")
+                       .with_stderr_contains("\
+[RUNNING] target[/]release[/]deps[/]foo-[..][EXE]")
+                       .with_stdout_contains("test bench_foo ... bench: [..]"));
+}
+
+#[test]
+fn bench_all_exclude() {
+    if !is_nightly() { return }
+
+    let p = project("foo")
+        .file("Cargo.toml", r#"
+            [project]
+            name = "foo"
+            version = "0.1.0"
+
+            [workspace]
+            members = ["bar", "baz"]
+        "#)
+        .file("src/main.rs", r#"
+            fn main() {}
+        "#)
+        .file("bar/Cargo.toml", r#"
+            [project]
+            name = "bar"
+            version = "0.1.0"
+        "#)
+        .file("bar/src/lib.rs", r#"
+            #![feature(test)]
+            #[cfg(test)]
+            extern crate test;
+
+            #[bench]
+            pub fn bar(b: &mut test::Bencher) {
+                b.iter(|| {});
+            }
+        "#)
+        .file("baz/Cargo.toml", r#"
+            [project]
+            name = "baz"
+            version = "0.1.0"
+        "#)
+        .file("baz/src/lib.rs", r#"
+            #[test]
+            pub fn baz() {
+                break_the_build();
+            }
+        "#)
+        .build();
+
+    assert_that(p.cargo("bench")
+                    .arg("--all")
+                    .arg("--exclude")
+                    .arg("baz"),
+                execs().with_status(0)
+                    .with_stdout_contains("\
 running 1 test
-test bench_bar ... bench:           0 ns/iter (+/- 0)
+test bar ... bench:           [..] ns/iter (+/- [..])"));
+}
+
+#[test]
+fn bench_all_virtual_manifest() {
+    if !is_nightly() { return }
+
+    let p = project("workspace")
+        .file("Cargo.toml", r#"
+            [workspace]
+            members = ["foo", "bar"]
+        "#)
+        .file("foo/Cargo.toml", r#"
+            [project]
+            name = "foo"
+            version = "0.1.0"
+        "#)
+        .file("foo/src/lib.rs", r#"
+            pub fn foo() {}
+        "#)
+        .file("foo/benches/foo.rs", r#"
+            #![feature(test)]
+            extern crate test;
+
+            use test::Bencher;
+
+            #[bench]
+            fn bench_foo(_: &mut Bencher) -> () { () }
+        "#)
+        .file("bar/Cargo.toml", r#"
+            [project]
+            name = "bar"
+            version = "0.1.0"
+        "#)
+        .file("bar/src/lib.rs", r#"
+            pub fn bar() {}
+        "#)
+        .file("bar/benches/bar.rs", r#"
+            #![feature(test)]
+            extern crate test;
+
+            use test::Bencher;
+
+            #[bench]
+            fn bench_bar(_: &mut Bencher) -> () { () }
+        "#)
+        .build();
+
+    // The order in which foo and bar are built is not guaranteed
+    assert_that(p.cargo("bench")
+                 .arg("--all"),
+                execs().with_status(0)
+                       .with_stderr_contains("\
+[RUNNING] target[/]release[/]deps[/]bar-[..][EXE]")
+                       .with_stdout_contains("test bench_bar ... bench: [..]")
+                       .with_stderr_contains("\
+[RUNNING] target[/]release[/]deps[/]foo-[..][EXE]")
+                       .with_stdout_contains("test bench_foo ... bench: [..]"));
+}
+
+// https://github.com/rust-lang/cargo/issues/4287
+#[test]
+fn legacy_bench_name() {
+    if !is_nightly() { return }
+
+    let p = project("foo")
+        .file("Cargo.toml", r#"
+            [project]
+            name = "foo"
+            version = "0.1.0"
+
+            [[bench]]
+            name = "bench"
+        "#)
+        .file("src/lib.rs", r#"
+            pub fn foo() {}
+        "#)
+        .file("src/bench.rs", r#"
+            #![feature(test)]
+            extern crate test;
+
+            use test::Bencher;
+
+            #[bench]
+            fn bench_foo(_: &mut Bencher) -> () { () }
+        "#)
+        .build();
+
+    assert_that(p.cargo("bench"), execs().with_status(0).with_stderr_contains("\
+[WARNING] path `[..]src[/]bench.rs` was erroneously implicitly accepted for benchmark `bench`,
+please set bench.path in Cargo.toml"));
+}
+
+#[test]
+fn bench_virtual_manifest_all_implied() {
+    if !is_nightly() { return }
+
+    let p = project("workspace")
+        .file("Cargo.toml", r#"
+            [workspace]
+            members = ["foo", "bar"]
+        "#)
+        .file("foo/Cargo.toml", r#"
+            [project]
+            name = "foo"
+            version = "0.1.0"
+        "#)
+        .file("foo/src/lib.rs", r#"
+            pub fn foo() {}
+        "#)
+        .file("foo/benches/foo.rs", r#"
+            #![feature(test)]
+            extern crate test;
+            use test::Bencher;
+            #[bench]
+            fn bench_foo(_: &mut Bencher) -> () { () }
+        "#)
+        .file("bar/Cargo.toml", r#"
+            [project]
+            name = "bar"
+            version = "0.1.0"
+        "#)
+        .file("bar/src/lib.rs", r#"
+            pub fn bar() {}
+        "#)
+        .file("bar/benches/bar.rs", r#"
+            #![feature(test)]
+            extern crate test;
+            use test::Bencher;
+            #[bench]
+            fn bench_bar(_: &mut Bencher) -> () { () }
+        "#)
+        .build();
+
+    // The order in which foo and bar are built is not guaranteed
 
-test result: ok. 0 passed; 0 failed; 0 ignored; 1 measured
-"));
+    assert_that(p.cargo("bench"),
+                execs().with_status(0)
+                       .with_stderr_contains("\
+[RUNNING] target[/]release[/]deps[/]bar-[..][EXE]")
+                       .with_stdout_contains("test bench_bar ... bench: [..]")
+                       .with_stderr_contains("\
+[RUNNING] target[/]release[/]deps[/]foo-[..][EXE]")
+                       .with_stdout_contains("test bench_foo ... bench: [..]"));
 }
