@@ -2,143 +2,14 @@ extern crate cargo;
 extern crate cargotest;
 extern crate hamcrest;
 
-use std::env;
-use std::process::Command;
-use std::sync::{Once, ONCE_INIT};
-use std::sync::atomic::{AtomicBool, ATOMIC_BOOL_INIT, Ordering};
-
 use cargo::util::process;
 use cargotest::{is_nightly, rustc_host};
-use cargotest::support::{project, execs, main_file, basic_bin_manifest};
+use cargotest::support::{project, execs, basic_bin_manifest, cross_compile};
 use hamcrest::{assert_that, existing_file};
 
-fn disabled() -> bool {
-    // First, disable if ./configure requested so
-    match env::var("CFG_DISABLE_CROSS_TESTS") {
-        Ok(ref s) if *s == "1" => return true,
-        _ => {}
-    }
-
-    // Right now the windows bots cannot cross compile due to the mingw setup,
-    // so we disable ourselves on all but macos/linux setups where the rustc
-    // install script ensures we have both architectures
-    if !(cfg!(target_os = "macos") ||
-         cfg!(target_os = "linux") ||
-         cfg!(target_env = "msvc")) {
-        return true;
-    }
-
-    // It's not particularly common to have a cross-compilation setup, so
-    // try to detect that before we fail a bunch of tests through no fault
-    // of the user.
-    static CAN_RUN_CROSS_TESTS: AtomicBool = ATOMIC_BOOL_INIT;
-    static CHECK: Once = ONCE_INIT;
-
-    let cross_target = alternate();
-
-    CHECK.call_once(|| {
-        let p = project("cross_test")
-            .file("Cargo.toml", &basic_bin_manifest("cross_test"))
-            .file("src/cross_test.rs", &main_file(r#""testing!""#, &[]));
-
-        let result = p.cargo_process("build")
-            .arg("--target").arg(&cross_target)
-            .exec_with_output();
-
-        if result.is_ok() {
-            CAN_RUN_CROSS_TESTS.store(true, Ordering::SeqCst);
-        }
-    });
-
-    if CAN_RUN_CROSS_TESTS.load(Ordering::SeqCst) {
-        // We were able to compile a simple project, so the user has the
-        // necessary std:: bits installed.  Therefore, tests should not
-        // be disabled.
-        return false;
-    }
-
-    // We can't compile a simple cross project.  We want to warn the user
-    // by failing a single test and having the remainder of the cross tests
-    // pass.  We don't use std::sync::Once here because panicing inside its
-    // call_once method would poison the Once instance, which is not what
-    // we want.
-    static HAVE_WARNED: AtomicBool = ATOMIC_BOOL_INIT;
-
-    if HAVE_WARNED.swap(true, Ordering::SeqCst) {
-        // We are some other test and somebody else is handling the warning.
-        // Just disable the current test.
-        return true;
-    }
-
-    // We are responsible for warning the user, which we do by panicing.
-    let rustup_available = Command::new("rustup").output().is_ok();
-
-    let linux_help = if cfg!(target_os = "linux") {
-        "
-
-You may need to install runtime libraries for your Linux distribution as well.".to_string()
-    } else {
-        "".to_string()
-    };
-
-    let rustup_help = if rustup_available {
-        format!("
-
-Alternatively, you can install the necessary libraries for cross-compilation with
-
-    rustup target add {}{}", cross_target, linux_help)
-    } else {
-        "".to_string()
-    };
-
-    panic!("Cannot cross compile to {}.
-
-This failure can be safely ignored. If you would prefer to not see this
-failure, you can set the environment variable CFG_DISABLE_CROSS_TESTS to \"1\".{}
-", cross_target, rustup_help);
-}
-
-fn alternate() -> String {
-    let platform = match env::consts::OS {
-        "linux" => "unknown-linux-gnu",
-        "macos" => "apple-darwin",
-        "windows" => "pc-windows-msvc",
-        _ => unreachable!(),
-    };
-    let arch = match env::consts::ARCH {
-        "x86" => "x86_64",
-        "x86_64" => "i686",
-        _ => unreachable!(),
-    };
-    format!("{}-{}", arch, platform)
-}
-
-fn alternate_arch() -> &'static str {
-    match env::consts::ARCH {
-        "x86" => "x86_64",
-        "x86_64" => "x86",
-        _ => unreachable!(),
-    }
-}
-
-fn host() -> String {
-    let platform = match env::consts::OS {
-        "linux" => "unknown-linux-gnu",
-        "macos" => "apple-darwin",
-        "windows" => "pc-windows-msvc",
-        _ => unreachable!(),
-    };
-    let arch = match env::consts::ARCH {
-        "x86" => "i686",
-        "x86_64" => "x86_64",
-        _ => unreachable!(),
-    };
-    format!("{}-{}", arch, platform)
-}
-
 #[test]
 fn simple_cross() {
-    if disabled() { return }
+    if cross_compile::disabled() { return }
 
     let p = project("foo")
         .file("Cargo.toml", r#"
@@ -152,16 +23,17 @@ fn simple_cross() {
             fn main() {{
                 assert_eq!(std::env::var("TARGET").unwrap(), "{}");
             }}
-        "#, alternate()))
+        "#, cross_compile::alternate()))
         .file("src/main.rs", &format!(r#"
             use std::env;
             fn main() {{
                 assert_eq!(env::consts::ARCH, "{}");
             }}
-        "#, alternate_arch()));
+        "#, cross_compile::alternate_arch()))
+        .build();
 
-    let target = alternate();
-    assert_that(p.cargo_process("build").arg("--target").arg(&target).arg("-v"),
+    let target = cross_compile::alternate();
+    assert_that(p.cargo("build").arg("--target").arg(&target).arg("-v"),
                 execs().with_status(0));
     assert_that(&p.target_bin(&target, "foo"), existing_file());
 
@@ -171,13 +43,13 @@ fn simple_cross() {
 
 #[test]
 fn simple_cross_config() {
-    if disabled() { return }
+    if cross_compile::disabled() { return }
 
     let p = project("foo")
         .file(".cargo/config", &format!(r#"
             [build]
             target = "{}"
-        "#, alternate()))
+        "#, cross_compile::alternate()))
         .file("Cargo.toml", r#"
             [package]
             name = "foo"
@@ -189,16 +61,17 @@ fn simple_cross_config() {
             fn main() {{
                 assert_eq!(std::env::var("TARGET").unwrap(), "{}");
             }}
-        "#, alternate()))
+        "#, cross_compile::alternate()))
         .file("src/main.rs", &format!(r#"
             use std::env;
             fn main() {{
                 assert_eq!(env::consts::ARCH, "{}");
             }}
-        "#, alternate_arch()));
+        "#, cross_compile::alternate_arch()))
+        .build();
 
-    let target = alternate();
-    assert_that(p.cargo_process("build").arg("-v"),
+    let target = cross_compile::alternate();
+    assert_that(p.cargo("build").arg("-v"),
                 execs().with_status(0));
     assert_that(&p.target_bin(&target, "foo"), existing_file());
 
@@ -208,7 +81,7 @@ fn simple_cross_config() {
 
 #[test]
 fn simple_deps() {
-    if disabled() { return }
+    if cross_compile::disabled() { return }
 
     let p = project("foo")
         .file("Cargo.toml", r#"
@@ -223,19 +96,20 @@ fn simple_deps() {
         .file("src/main.rs", r#"
             extern crate bar;
             fn main() { bar::bar(); }
-        "#);
-    let p2 = project("bar")
+        "#)
+        .build();
+    let _p2 = project("bar")
         .file("Cargo.toml", r#"
             [package]
             name = "bar"
             version = "0.0.1"
             authors = []
         "#)
-        .file("src/lib.rs", "pub fn bar() {}");
-    p2.build();
+        .file("src/lib.rs", "pub fn bar() {}")
+        .build();
 
-    let target = alternate();
-    assert_that(p.cargo_process("build").arg("--target").arg(&target),
+    let target = cross_compile::alternate();
+    assert_that(p.cargo("build").arg("--target").arg(&target),
                 execs().with_status(0));
     assert_that(&p.target_bin(&target, "foo"), existing_file());
 
@@ -245,7 +119,7 @@ fn simple_deps() {
 
 #[test]
 fn plugin_deps() {
-    if disabled() { return }
+    if cross_compile::disabled() { return }
     if !is_nightly() { return }
 
     let foo = project("foo")
@@ -268,8 +142,9 @@ fn plugin_deps() {
             fn main() {
                 assert_eq!(bar!(), baz::baz());
             }
-        "#);
-    let bar = project("bar")
+        "#)
+        .build();
+    let _bar = project("bar")
         .file("Cargo.toml", r#"
             [package]
             name = "bar"
@@ -289,7 +164,9 @@ fn plugin_deps() {
             use rustc_plugin::Registry;
             use syntax::tokenstream::TokenTree;
             use syntax::codemap::Span;
+            use syntax::ast::*;
             use syntax::ext::base::{ExtCtxt, MacEager, MacResult};
+            use syntax::ext::build::AstBuilder;
 
             #[plugin_registrar]
             pub fn foo(reg: &mut Registry) {
@@ -298,22 +175,22 @@ fn plugin_deps() {
 
             fn expand_bar(cx: &mut ExtCtxt, sp: Span, tts: &[TokenTree])
                           -> Box<MacResult + 'static> {
-                MacEager::expr(quote_expr!(cx, 1))
+                MacEager::expr(cx.expr_lit(sp, LitKind::Int(1, LitIntType::Unsuffixed)))
             }
-        "#);
-    let baz = project("baz")
+        "#)
+        .build();
+    let _baz = project("baz")
         .file("Cargo.toml", r#"
             [package]
             name = "baz"
             version = "0.0.1"
             authors = []
         "#)
-        .file("src/lib.rs", "pub fn baz() -> i32 { 1 }");
-    bar.build();
-    baz.build();
+        .file("src/lib.rs", "pub fn baz() -> i32 { 1 }")
+        .build();
 
-    let target = alternate();
-    assert_that(foo.cargo_process("build").arg("--target").arg(&target),
+    let target = cross_compile::alternate();
+    assert_that(foo.cargo("build").arg("--target").arg(&target),
                 execs().with_status(0));
     assert_that(&foo.target_bin(&target, "foo"), existing_file());
 
@@ -323,7 +200,7 @@ fn plugin_deps() {
 
 #[test]
 fn plugin_to_the_max() {
-    if disabled() { return }
+    if cross_compile::disabled() { return }
     if !is_nightly() { return }
 
     let foo = project("foo")
@@ -346,8 +223,9 @@ fn plugin_to_the_max() {
             fn main() {
                 assert_eq!(bar!(), baz::baz());
             }
-        "#);
-    let bar = project("bar")
+        "#)
+        .build();
+    let _bar = project("bar")
         .file("Cargo.toml", r#"
             [package]
             name = "bar"
@@ -371,7 +249,10 @@ fn plugin_to_the_max() {
             use rustc_plugin::Registry;
             use syntax::tokenstream::TokenTree;
             use syntax::codemap::Span;
+            use syntax::ast::*;
             use syntax::ext::base::{ExtCtxt, MacEager, MacResult};
+            use syntax::ext::build::AstBuilder;
+            use syntax::ptr::P;
 
             #[plugin_registrar]
             pub fn foo(reg: &mut Registry) {
@@ -380,22 +261,24 @@ fn plugin_to_the_max() {
 
             fn expand_bar(cx: &mut ExtCtxt, sp: Span, tts: &[TokenTree])
                           -> Box<MacResult + 'static> {
-                MacEager::expr(quote_expr!(cx, baz::baz()))
+                let bar = Ident::from_str("baz");
+                let path = cx.path(sp, vec![bar.clone(), bar]);
+                MacEager::expr(cx.expr_call(sp, cx.expr_path(path), vec![]))
             }
-        "#);
-    let baz = project("baz")
+        "#)
+        .build();
+    let _baz = project("baz")
         .file("Cargo.toml", r#"
             [package]
             name = "baz"
             version = "0.0.1"
             authors = []
         "#)
-        .file("src/lib.rs", "pub fn baz() -> i32 { 1 }");
-    bar.build();
-    baz.build();
+        .file("src/lib.rs", "pub fn baz() -> i32 { 1 }")
+        .build();
 
-    let target = alternate();
-    assert_that(foo.cargo_process("build").arg("--target").arg(&target).arg("-v"),
+    let target = cross_compile::alternate();
+    assert_that(foo.cargo("build").arg("--target").arg(&target).arg("-v"),
                 execs().with_status(0));
     println!("second");
     assert_that(foo.cargo("build").arg("-v")
@@ -409,9 +292,9 @@ fn plugin_to_the_max() {
 
 #[test]
 fn linker_and_ar() {
-    if disabled() { return }
+    if cross_compile::disabled() { return }
 
-    let target = alternate();
+    let target = cross_compile::alternate();
     let p = project("foo")
         .file(".cargo/config", &format!(r#"
             [target.{}]
@@ -424,9 +307,10 @@ fn linker_and_ar() {
             fn main() {{
                 assert_eq!(env::consts::ARCH, "{}");
             }}
-        "#, alternate_arch()));
+        "#, cross_compile::alternate_arch()))
+        .build();
 
-    assert_that(p.cargo_process("build").arg("--target").arg(&target)
+    assert_that(p.cargo("build").arg("--target").arg(&target)
                                               .arg("-v"),
                 execs().with_status(101)
                        .with_stderr_contains(&format!("\
@@ -448,7 +332,7 @@ fn linker_and_ar() {
 
 #[test]
 fn plugin_with_extra_dylib_dep() {
-    if disabled() { return }
+    if cross_compile::disabled() { return }
     if !is_nightly() { return }
 
     let foo = project("foo")
@@ -466,8 +350,9 @@ fn plugin_with_extra_dylib_dep() {
             #![plugin(bar)]
 
             fn main() {}
-        "#);
-    let bar = project("bar")
+        "#)
+        .build();
+    let _bar = project("bar")
         .file("Cargo.toml", r#"
             [package]
             name = "bar"
@@ -493,8 +378,9 @@ fn plugin_with_extra_dylib_dep() {
             pub fn foo(reg: &mut Registry) {
                 println!("{}", baz::baz());
             }
-        "#);
-    let baz = project("baz")
+        "#)
+        .build();
+    let _baz = project("baz")
         .file("Cargo.toml", r#"
             [package]
             name = "baz"
@@ -505,18 +391,17 @@ fn plugin_with_extra_dylib_dep() {
             name = "baz"
             crate_type = ["dylib"]
         "#)
-        .file("src/lib.rs", "pub fn baz() -> i32 { 1 }");
-    bar.build();
-    baz.build();
+        .file("src/lib.rs", "pub fn baz() -> i32 { 1 }")
+        .build();
 
-    let target = alternate();
-    assert_that(foo.cargo_process("build").arg("--target").arg(&target),
+    let target = cross_compile::alternate();
+    assert_that(foo.cargo("build").arg("--target").arg(&target),
                 execs().with_status(0));
 }
 
 #[test]
 fn cross_tests() {
-    if disabled() { return }
+    if cross_compile::disabled() { return }
 
     let p = project("foo")
         .file("Cargo.toml", r#"
@@ -536,15 +421,16 @@ fn cross_tests() {
                 assert_eq!(env::consts::ARCH, "{}");
             }}
             #[test] fn test() {{ main() }}
-        "#, alternate_arch()))
+        "#, cross_compile::alternate_arch()))
         .file("src/lib.rs", &format!(r#"
             use std::env;
             pub fn foo() {{ assert_eq!(env::consts::ARCH, "{}"); }}
             #[test] fn test_foo() {{ foo() }}
-        "#, alternate_arch()));
+        "#, cross_compile::alternate_arch()))
+        .build();
 
-    let target = alternate();
-    assert_that(p.cargo_process("test").arg("--target").arg(&target),
+    let target = cross_compile::alternate();
+    assert_that(p.cargo("test").arg("--target").arg(&target),
                 execs().with_status(0)
                        .with_stderr(&format!("\
 [COMPILING] foo v0.0.0 ({foo})
@@ -557,7 +443,7 @@ fn cross_tests() {
 
 #[test]
 fn no_cross_doctests() {
-    if disabled() { return }
+    if cross_compile::disabled() { return }
 
     let p = project("foo")
         .file("Cargo.toml", r#"
@@ -571,8 +457,8 @@ fn no_cross_doctests() {
             //! extern crate foo;
             //! assert!(true);
             //! ```
-        "#);
-    p.build();
+        "#)
+        .build();
 
     let host_output = format!("\
 [COMPILING] foo v0.0.0 ({foo})
@@ -587,7 +473,7 @@ fn no_cross_doctests() {
                        .with_stderr(&host_output));
 
     println!("b");
-    let target = host();
+    let target = cross_compile::host();
     assert_that(p.cargo("test").arg("--target").arg(&target),
                 execs().with_status(0)
                        .with_stderr(&format!("\
@@ -598,7 +484,7 @@ fn no_cross_doctests() {
 ", foo = p.url(), triple = target)));
 
     println!("c");
-    let target = alternate();
+    let target = cross_compile::alternate();
     assert_that(p.cargo("test").arg("--target").arg(&target),
                 execs().with_status(0)
                        .with_stderr(&format!("\
@@ -610,7 +496,7 @@ fn no_cross_doctests() {
 
 #[test]
 fn simple_cargo_run() {
-    if disabled() { return }
+    if cross_compile::disabled() { return }
 
     let p = project("foo")
         .file("Cargo.toml", r#"
@@ -624,18 +510,19 @@ fn simple_cargo_run() {
             fn main() {{
                 assert_eq!(env::consts::ARCH, "{}");
             }}
-        "#, alternate_arch()));
+        "#, cross_compile::alternate_arch()))
+        .build();
 
-    let target = alternate();
-    assert_that(p.cargo_process("run").arg("--target").arg(&target),
+    let target = cross_compile::alternate();
+    assert_that(p.cargo("run").arg("--target").arg(&target),
                 execs().with_status(0));
 }
 
 #[test]
 fn cross_with_a_build_script() {
-    if disabled() { return }
+    if cross_compile::disabled() { return }
 
-    let target = alternate();
+    let target = cross_compile::alternate();
     let p = project("foo")
         .file("Cargo.toml", r#"
             [package]
@@ -664,9 +551,10 @@ fn cross_with_a_build_script() {
                 assert_eq!(path.file_name().unwrap().to_str().unwrap(), "target");
             }}
         "#, target))
-        .file("src/main.rs", "fn main() {}");
+        .file("src/main.rs", "fn main() {}")
+        .build();
 
-    assert_that(p.cargo_process("build").arg("--target").arg(&target).arg("-v"),
+    assert_that(p.cargo("build").arg("--target").arg(&target).arg("-v"),
                 execs().with_status(0)
                        .with_stderr(&format!("\
 [COMPILING] foo v0.0.0 (file://[..])
@@ -680,9 +568,9 @@ fn cross_with_a_build_script() {
 
 #[test]
 fn build_script_needed_for_host_and_target() {
-    if disabled() { return }
+    if cross_compile::disabled() { return }
 
-    let target = alternate();
+    let target = cross_compile::alternate();
     let host = rustc_host();
     let p = project("foo")
         .file("Cargo.toml", r#"
@@ -738,9 +626,10 @@ fn build_script_needed_for_host_and_target() {
             #[allow(unused_extern_crates)]
             extern crate d1;
             pub fn d2() { d1::d1(); }
-        ");
+        ")
+        .build();
 
-    assert_that(p.cargo_process("build").arg("--target").arg(&target).arg("-v"),
+    assert_that(p.cargo("build").arg("--target").arg(&target).arg("-v"),
                 execs().with_status(0)
                        .with_stderr_contains(&format!("\
 [COMPILING] d1 v0.0.0 ({url}/d1)", url = p.url()))
@@ -769,7 +658,7 @@ fn build_script_needed_for_host_and_target() {
 
 #[test]
 fn build_deps_for_the_right_arch() {
-    if disabled() { return }
+    if cross_compile::disabled() { return }
 
     let p = project("foo")
         .file("Cargo.toml", r#"
@@ -802,16 +691,17 @@ fn build_deps_for_the_right_arch() {
             path = "../d1"
         "#)
         .file("d2/build.rs", "extern crate d1; fn main() {}")
-        .file("d2/src/lib.rs", "");
+        .file("d2/src/lib.rs", "")
+        .build();
 
-    let target = alternate();
-    assert_that(p.cargo_process("build").arg("--target").arg(&target).arg("-v"),
+    let target = cross_compile::alternate();
+    assert_that(p.cargo("build").arg("--target").arg(&target).arg("-v"),
                 execs().with_status(0));
 }
 
 #[test]
 fn build_script_only_host() {
-    if disabled() { return }
+    if cross_compile::disabled() { return }
 
     let p = project("foo")
         .file("Cargo.toml", r#"
@@ -844,16 +734,17 @@ fn build_script_only_host() {
                                            .contains("target/debug/build/d1-"),
                         "bad: {:?}", env::var("OUT_DIR"));
             }
-        "#);
+        "#)
+        .build();
 
-    let target = alternate();
-    assert_that(p.cargo_process("build").arg("--target").arg(&target).arg("-v"),
+    let target = cross_compile::alternate();
+    assert_that(p.cargo("build").arg("--target").arg(&target).arg("-v"),
                 execs().with_status(0));
 }
 
 #[test]
 fn plugin_build_script_right_arch() {
-    if disabled() { return }
+    if cross_compile::disabled() { return }
     let p = project("foo")
         .file("Cargo.toml", r#"
             [package]
@@ -867,9 +758,10 @@ fn plugin_build_script_right_arch() {
             plugin = true
         "#)
         .file("build.rs", "fn main() {}")
-        .file("src/lib.rs", "");
+        .file("src/lib.rs", "")
+        .build();
 
-    assert_that(p.cargo_process("build").arg("-v").arg("--target").arg(alternate()),
+    assert_that(p.cargo("build").arg("-v").arg("--target").arg(cross_compile::alternate()),
                 execs().with_status(0)
                        .with_stderr("\
 [COMPILING] foo v0.0.1 ([..])
@@ -882,9 +774,9 @@ fn plugin_build_script_right_arch() {
 
 #[test]
 fn build_script_with_platform_specific_dependencies() {
-    if disabled() { return }
+    if cross_compile::disabled() { return }
 
-    let target = alternate();
+    let target = cross_compile::alternate();
     let host = rustc_host();
     let p = project("foo")
         .file("Cargo.toml", r#"
@@ -922,9 +814,10 @@ fn build_script_with_platform_specific_dependencies() {
             version = "0.0.0"
             authors = []
         "#)
-        .file("d2/src/lib.rs", "");
+        .file("d2/src/lib.rs", "")
+        .build();
 
-    assert_that(p.cargo_process("build").arg("-v").arg("--target").arg(&target),
+    assert_that(p.cargo("build").arg("-v").arg("--target").arg(&target),
                 execs().with_status(0)
                        .with_stderr(&format!("\
 [COMPILING] d2 v0.0.0 ([..])
@@ -941,9 +834,9 @@ fn build_script_with_platform_specific_dependencies() {
 
 #[test]
 fn platform_specific_dependencies_do_not_leak() {
-    if disabled() { return }
+    if cross_compile::disabled() { return }
 
-    let target = alternate();
+    let target = cross_compile::alternate();
     let host = rustc_host();
     let p = project("foo")
         .file("Cargo.toml", r#"
@@ -977,9 +870,10 @@ fn platform_specific_dependencies_do_not_leak() {
             version = "0.0.0"
             authors = []
         "#)
-        .file("d2/src/lib.rs", "");
+        .file("d2/src/lib.rs", "")
+        .build();
 
-    assert_that(p.cargo_process("build").arg("-v").arg("--target").arg(&target),
+    assert_that(p.cargo("build").arg("-v").arg("--target").arg(&target),
                 execs().with_status(101)
                        .with_stderr_contains("\
 [..] can't find crate for `d2`[..]"));
@@ -987,9 +881,9 @@ fn platform_specific_dependencies_do_not_leak() {
 
 #[test]
 fn platform_specific_variables_reflected_in_build_scripts() {
-    if disabled() { return }
+    if cross_compile::disabled() { return }
 
-    let target = alternate();
+    let target = cross_compile::alternate();
     let host = rustc_host();
     let p = project("foo")
         .file("Cargo.toml", &format!(r#"
@@ -1046,8 +940,8 @@ fn platform_specific_variables_reflected_in_build_scripts() {
         .file("d2/build.rs", r#"
             fn main() { println!("cargo:val=1") }
         "#)
-        .file("d2/src/lib.rs", "");
-    p.build();
+        .file("d2/src/lib.rs", "")
+        .build();
 
     assert_that(p.cargo("build").arg("-v"), execs().with_status(0));
     assert_that(p.cargo("build").arg("-v").arg("--target").arg(&target),
@@ -1056,9 +950,9 @@ fn platform_specific_variables_reflected_in_build_scripts() {
 
 #[test]
 fn cross_test_dylib() {
-    if disabled() { return }
+    if cross_compile::disabled() { return }
 
-    let target = alternate();
+    let target = cross_compile::alternate();
 
     let p = project("foo")
         .file("Cargo.toml", r#"
@@ -1103,9 +997,10 @@ fn cross_test_dylib() {
              pub fn baz() {{
                 assert_eq!(env::consts::ARCH, "{}");
             }}
-        "#, alternate_arch()));
+        "#, cross_compile::alternate_arch()))
+        .build();
 
-    assert_that(p.cargo_process("test").arg("--target").arg(&target),
+    assert_that(p.cargo("test").arg("--target").arg(&target),
                 execs().with_status(0)
                        .with_stderr(&format!("\
 [COMPILING] bar v0.0.1 ({dir}/bar)
@@ -1113,7 +1008,7 @@ fn cross_test_dylib() {
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 [RUNNING] target[/]{arch}[/]debug[/]deps[/]foo-[..][EXE]
 [RUNNING] target[/]{arch}[/]debug[/]deps[/]test-[..][EXE]",
-                        dir = p.url(), arch = alternate()))
+                        dir = p.url(), arch = cross_compile::alternate()))
                        .with_stdout_contains_n("test foo ... ok", 2));
 
 }
