@@ -1,48 +1,23 @@
-#[macro_use]
 extern crate cargotest;
 extern crate flate2;
 extern crate hamcrest;
 extern crate tar;
-extern crate url;
 
 use std::io::prelude::*;
-use std::fs::{self, File};
+use std::fs::File;
 use std::io::SeekFrom;
-use std::path::PathBuf;
 
+use cargotest::ChannelChanger;
 use cargotest::support::git::repo;
 use cargotest::support::paths;
-use cargotest::support::{project, execs};
+use cargotest::support::{project, execs, publish};
 use flate2::read::GzDecoder;
 use hamcrest::assert_that;
 use tar::Archive;
-use url::Url;
-
-fn registry_path() -> PathBuf { paths::root().join("registry") }
-fn registry() -> Url { Url::from_file_path(&*registry_path()).ok().unwrap() }
-fn upload_path() -> PathBuf { paths::root().join("upload") }
-fn upload() -> Url { Url::from_file_path(&*upload_path()).ok().unwrap() }
-
-fn setup() {
-    let config = paths::root().join(".cargo/config");
-    t!(fs::create_dir_all(config.parent().unwrap()));
-    t!(t!(File::create(&config)).write_all(br#"
-        [registry]
-            token = "api-token"
-    "#));
-    t!(fs::create_dir_all(&upload_path().join("api/v1/crates")));
-
-    repo(&registry_path())
-        .file("config.json", &format!(r#"{{
-            "dl": "{0}",
-            "api": "{0}"
-        }}"#, upload()))
-        .build();
-}
 
 #[test]
 fn simple() {
-    setup();
+    publish::setup();
 
     let p = project("foo")
         .file("Cargo.toml", r#"
@@ -53,10 +28,11 @@ fn simple() {
             license = "MIT"
             description = "foo"
         "#)
-        .file("src/main.rs", "fn main() {}");
+        .file("src/main.rs", "fn main() {}")
+        .build();
 
-    assert_that(p.cargo_process("publish").arg("--no-verify")
-                 .arg("--index").arg(registry().to_string()),
+    assert_that(p.cargo("publish").arg("--no-verify")
+                 .arg("--index").arg(publish::registry().to_string()),
                 execs().with_status(0).with_stderr(&format!("\
 [UPDATING] registry `{reg}`
 [WARNING] manifest has no documentation, [..]
@@ -65,9 +41,9 @@ See [..]
 [UPLOADING] foo v0.0.1 ({dir})
 ",
         dir = p.url(),
-        reg = registry())));
+        reg = publish::registry())));
 
-    let mut f = File::open(&upload_path().join("api/v1/crates/new")).unwrap();
+    let mut f = File::open(&publish::upload_path().join("api/v1/crates/new")).unwrap();
     // Skip the metadata payload and the size of the tarball
     let mut sz = [0; 4];
     assert_eq!(f.read(&mut sz).unwrap(), 4);
@@ -78,8 +54,8 @@ See [..]
     f.seek(SeekFrom::Current(sz as i64 + 4)).unwrap();
 
     // Verify the tarball
-    let mut rdr = GzDecoder::new(f).unwrap();
-    assert_eq!(rdr.header().filename().unwrap(), "foo-0.0.1.crate".as_bytes());
+    let mut rdr = GzDecoder::new(f);
+    assert_eq!(rdr.header().unwrap().filename().unwrap(), b"foo-0.0.1.crate");
     let mut contents = Vec::new();
     rdr.read_to_end(&mut contents).unwrap();
     let mut ar = Archive::new(&contents[..]);
@@ -98,7 +74,7 @@ See [..]
 // remove once it has been decided --host can be removed
 #[test]
 fn simple_with_host() {
-    setup();
+    publish::setup();
 
     let p = project("foo")
         .file("Cargo.toml", r#"
@@ -109,10 +85,11 @@ fn simple_with_host() {
             license = "MIT"
             description = "foo"
         "#)
-        .file("src/main.rs", "fn main() {}");
+        .file("src/main.rs", "fn main() {}")
+        .build();
 
-    assert_that(p.cargo_process("publish").arg("--no-verify")
-                 .arg("--host").arg(registry().to_string()),
+    assert_that(p.cargo("publish").arg("--no-verify")
+                 .arg("--host").arg(publish::registry().to_string()),
                 execs().with_status(0).with_stderr(&format!("\
 [WARNING] The flag '--host' is no longer valid.
 
@@ -131,9 +108,9 @@ See [..]
 [UPLOADING] foo v0.0.1 ({dir})
 ",
         dir = p.url(),
-        reg = registry())));
+        reg = publish::registry())));
 
-    let mut f = File::open(&upload_path().join("api/v1/crates/new")).unwrap();
+    let mut f = File::open(&publish::upload_path().join("api/v1/crates/new")).unwrap();
     // Skip the metadata payload and the size of the tarball
     let mut sz = [0; 4];
     assert_eq!(f.read(&mut sz).unwrap(), 4);
@@ -144,8 +121,8 @@ See [..]
     f.seek(SeekFrom::Current(sz as i64 + 4)).unwrap();
 
     // Verify the tarball
-    let mut rdr = GzDecoder::new(f).unwrap();
-    assert_eq!(rdr.header().filename().unwrap(), "foo-0.0.1.crate".as_bytes());
+    let mut rdr = GzDecoder::new(f);
+    assert_eq!(rdr.header().unwrap().filename().unwrap(), "foo-0.0.1.crate".as_bytes());
     let mut contents = Vec::new();
     rdr.read_to_end(&mut contents).unwrap();
     let mut ar = Archive::new(&contents[..]);
@@ -164,7 +141,7 @@ See [..]
 // remove once it has been decided --host can be removed
 #[test]
 fn simple_with_index_and_host() {
-    setup();
+    publish::setup();
 
     let p = project("foo")
         .file("Cargo.toml", r#"
@@ -175,11 +152,12 @@ fn simple_with_index_and_host() {
             license = "MIT"
             description = "foo"
         "#)
-        .file("src/main.rs", "fn main() {}");
+        .file("src/main.rs", "fn main() {}")
+        .build();
 
-    assert_that(p.cargo_process("publish").arg("--no-verify")
-                 .arg("--index").arg(registry().to_string())
-                 .arg("--host").arg(registry().to_string()),
+    assert_that(p.cargo("publish").arg("--no-verify")
+                 .arg("--index").arg(publish::registry().to_string())
+                 .arg("--host").arg(publish::registry().to_string()),
                 execs().with_status(0).with_stderr(&format!("\
 [WARNING] The flag '--host' is no longer valid.
 
@@ -198,9 +176,9 @@ See [..]
 [UPLOADING] foo v0.0.1 ({dir})
 ",
         dir = p.url(),
-        reg = registry())));
+        reg = publish::registry())));
 
-    let mut f = File::open(&upload_path().join("api/v1/crates/new")).unwrap();
+    let mut f = File::open(&publish::upload_path().join("api/v1/crates/new")).unwrap();
     // Skip the metadata payload and the size of the tarball
     let mut sz = [0; 4];
     assert_eq!(f.read(&mut sz).unwrap(), 4);
@@ -211,8 +189,8 @@ See [..]
     f.seek(SeekFrom::Current(sz as i64 + 4)).unwrap();
 
     // Verify the tarball
-    let mut rdr = GzDecoder::new(f).unwrap();
-    assert_eq!(rdr.header().filename().unwrap(), "foo-0.0.1.crate".as_bytes());
+    let mut rdr = GzDecoder::new(f);
+    assert_eq!(rdr.header().unwrap().filename().unwrap(), "foo-0.0.1.crate".as_bytes());
     let mut contents = Vec::new();
     rdr.read_to_end(&mut contents).unwrap();
     let mut ar = Archive::new(&contents[..]);
@@ -229,7 +207,7 @@ See [..]
 
 #[test]
 fn git_deps() {
-    setup();
+    publish::setup();
 
     let p = project("foo")
         .file("Cargo.toml", r#"
@@ -243,10 +221,11 @@ fn git_deps() {
             [dependencies.foo]
             git = "git://path/to/nowhere"
         "#)
-        .file("src/main.rs", "fn main() {}");
+        .file("src/main.rs", "fn main() {}")
+        .build();
 
-    assert_that(p.cargo_process("publish").arg("-v").arg("--no-verify")
-                 .arg("--index").arg(registry().to_string()),
+    assert_that(p.cargo("publish").arg("-v").arg("--no-verify")
+                 .arg("--index").arg(publish::registry().to_string()),
                 execs().with_status(101).with_stderr("\
 [UPDATING] registry [..]
 [ERROR] crates cannot be published to crates.io with dependencies sourced from \
@@ -259,7 +238,7 @@ repository and specify it with a path and version\n\
 
 #[test]
 fn path_dependency_no_version() {
-    setup();
+    publish::setup();
 
     let p = project("foo")
         .file("Cargo.toml", r#"
@@ -280,10 +259,11 @@ fn path_dependency_no_version() {
             version = "0.0.1"
             authors = []
         "#)
-        .file("bar/src/lib.rs", "");
+        .file("bar/src/lib.rs", "")
+        .build();
 
-    assert_that(p.cargo_process("publish")
-                 .arg("--index").arg(registry().to_string()),
+    assert_that(p.cargo("publish")
+                 .arg("--index").arg(publish::registry().to_string()),
                 execs().with_status(101).with_stderr("\
 [UPDATING] registry [..]
 [ERROR] all path dependencies must have a version specified when publishing.
@@ -293,7 +273,7 @@ dependency `bar` does not specify a version
 
 #[test]
 fn unpublishable_crate() {
-    setup();
+    publish::setup();
 
     let p = project("foo")
         .file("Cargo.toml", r#"
@@ -305,10 +285,11 @@ fn unpublishable_crate() {
             description = "foo"
             publish = false
         "#)
-        .file("src/main.rs", "fn main() {}");
+        .file("src/main.rs", "fn main() {}")
+        .build();
 
-    assert_that(p.cargo_process("publish")
-                 .arg("--index").arg(registry().to_string()),
+    assert_that(p.cargo("publish")
+                 .arg("--index").arg(publish::registry().to_string()),
                 execs().with_status(101).with_stderr("\
 [ERROR] some crates cannot be published.
 `foo` is marked as unpublishable
@@ -317,12 +298,12 @@ fn unpublishable_crate() {
 
 #[test]
 fn dont_publish_dirty() {
-    setup();
+    publish::setup();
     let p = project("foo")
-        .file("bar", "");
-    p.build();
+        .file("bar", "")
+        .build();
 
-    repo(&paths::root().join("foo"))
+    let _ = repo(&paths::root().join("foo"))
         .file("Cargo.toml", r#"
             [project]
             name = "foo"
@@ -338,7 +319,7 @@ fn dont_publish_dirty() {
         .build();
 
     assert_that(p.cargo("publish")
-                 .arg("--index").arg(registry().to_string()),
+                 .arg("--index").arg(publish::registry().to_string()),
                 execs().with_status(101).with_stderr("\
 [UPDATING] registry `[..]`
 error: 1 files in the working directory contain changes that were not yet \
@@ -352,12 +333,11 @@ to proceed despite this, pass the `--allow-dirty` flag
 
 #[test]
 fn publish_clean() {
-    setup();
+    publish::setup();
 
-    let p = project("foo");
-    p.build();
+    let p = project("foo").build();
 
-    repo(&paths::root().join("foo"))
+    let _ = repo(&paths::root().join("foo"))
         .file("Cargo.toml", r#"
             [project]
             name = "foo"
@@ -373,19 +353,19 @@ fn publish_clean() {
         .build();
 
     assert_that(p.cargo("publish")
-                 .arg("--index").arg(registry().to_string()),
+                 .arg("--index").arg(publish::registry().to_string()),
                 execs().with_status(0));
 }
 
 #[test]
 fn publish_in_sub_repo() {
-    setup();
+    publish::setup();
 
     let p = project("foo")
-        .file("baz", "");
-    p.build();
+        .file("baz", "")
+        .build();
 
-    repo(&paths::root().join("foo"))
+    let _ = repo(&paths::root().join("foo"))
         .file("bar/Cargo.toml", r#"
             [project]
             name = "foo"
@@ -401,19 +381,19 @@ fn publish_in_sub_repo() {
         .build();
 
     assert_that(p.cargo("publish").cwd(p.root().join("bar"))
-                 .arg("--index").arg(registry().to_string()),
+                 .arg("--index").arg(publish::registry().to_string()),
                 execs().with_status(0));
 }
 
 #[test]
 fn publish_when_ignored() {
-    setup();
+    publish::setup();
 
     let p = project("foo")
-        .file("baz", "");
-    p.build();
+        .file("baz", "")
+        .build();
 
-    repo(&paths::root().join("foo"))
+    let _ = repo(&paths::root().join("foo"))
         .file("Cargo.toml", r#"
             [project]
             name = "foo"
@@ -430,19 +410,19 @@ fn publish_when_ignored() {
         .build();
 
     assert_that(p.cargo("publish")
-                 .arg("--index").arg(registry().to_string()),
+                 .arg("--index").arg(publish::registry().to_string()),
                 execs().with_status(0));
 }
 
 #[test]
 fn ignore_when_crate_ignored() {
-    setup();
+    publish::setup();
 
     let p = project("foo")
-        .file("bar/baz", "");
-    p.build();
+        .file("bar/baz", "")
+        .build();
 
-    repo(&paths::root().join("foo"))
+    let _ = repo(&paths::root().join("foo"))
         .file(".gitignore", "bar")
         .nocommit_file("bar/Cargo.toml", r#"
             [project]
@@ -457,19 +437,19 @@ fn ignore_when_crate_ignored() {
         "#)
         .nocommit_file("bar/src/main.rs", "fn main() {}");
     assert_that(p.cargo("publish").cwd(p.root().join("bar"))
-                 .arg("--index").arg(registry().to_string()),
+                 .arg("--index").arg(publish::registry().to_string()),
                 execs().with_status(0));
 }
 
 #[test]
 fn new_crate_rejected() {
-    setup();
+    publish::setup();
 
     let p = project("foo")
-        .file("baz", "");
-    p.build();
+        .file("baz", "")
+        .build();
 
-    repo(&paths::root().join("foo"))
+    let _ = repo(&paths::root().join("foo"))
         .nocommit_file("Cargo.toml", r#"
             [project]
             name = "foo"
@@ -483,13 +463,13 @@ fn new_crate_rejected() {
         "#)
         .nocommit_file("src/main.rs", "fn main() {}");
     assert_that(p.cargo("publish")
-                 .arg("--index").arg(registry().to_string()),
+                 .arg("--index").arg(publish::registry().to_string()),
                 execs().with_status(101));
 }
 
 #[test]
 fn dry_run() {
-    setup();
+    publish::setup();
 
     let p = project("foo")
         .file("Cargo.toml", r#"
@@ -500,10 +480,11 @@ fn dry_run() {
             license = "MIT"
             description = "foo"
         "#)
-        .file("src/main.rs", "fn main() {}");
+        .file("src/main.rs", "fn main() {}")
+        .build();
 
-    assert_that(p.cargo_process("publish").arg("--dry-run")
-                 .arg("--index").arg(registry().to_string()),
+    assert_that(p.cargo("publish").arg("--dry-run")
+                 .arg("--index").arg(publish::registry().to_string()),
                 execs().with_status(0).with_stderr(&format!("\
 [UPDATING] registry `[..]`
 [WARNING] manifest has no documentation, [..]
@@ -518,5 +499,150 @@ See [..]
         dir = p.url())));
 
     // Ensure the API request wasn't actually made
-    assert!(!upload_path().join("api/v1/crates/new").exists());
+    assert!(!publish::upload_path().join("api/v1/crates/new").exists());
+}
+
+#[test]
+fn block_publish_feature_not_enabled() {
+    publish::setup();
+
+    let p = project("foo")
+        .file("Cargo.toml", r#"
+            [project]
+            name = "foo"
+            version = "0.0.1"
+            authors = []
+            license = "MIT"
+            description = "foo"
+            publish = [
+                "test"
+            ]
+        "#)
+        .file("src/main.rs", "fn main() {}")
+        .build();
+
+    assert_that(p.cargo("publish").masquerade_as_nightly_cargo()
+                 .arg("--registry").arg("alternative").arg("-Zunstable-options"),
+                execs().with_status(101).with_stderr("\
+error: failed to parse manifest at `[..]`
+
+Caused by:
+  the `publish` manifest key is unstable for anything other than a value of true or false
+
+Caused by:
+  feature `alternative-registries` is required
+
+consider adding `cargo-features = [\"alternative-registries\"]` to the manifest
+"));
+}
+
+#[test]
+fn registry_not_in_publish_list() {
+    publish::setup();
+
+    let p = project("foo")
+        .file("Cargo.toml", r#"
+            cargo-features = ["alternative-registries"]
+
+            [project]
+            name = "foo"
+            version = "0.0.1"
+            authors = []
+            license = "MIT"
+            description = "foo"
+            publish = [
+                "test"
+            ]
+        "#)
+        .file("src/main.rs", "fn main() {}")
+        .build();
+
+    assert_that(p.cargo("publish").masquerade_as_nightly_cargo()
+                 .arg("--registry").arg("alternative").arg("-Zunstable-options"),
+                execs().with_status(101).with_stderr("\
+[ERROR] some crates cannot be published.
+`foo` is marked as unpublishable
+"));
+}
+
+#[test]
+fn publish_empty_list() {
+    publish::setup();
+
+    let p = project("foo")
+        .file("Cargo.toml", r#"
+            cargo-features = ["alternative-registries"]
+
+            [project]
+            name = "foo"
+            version = "0.0.1"
+            authors = []
+            license = "MIT"
+            description = "foo"
+            publish = []
+        "#)
+        .file("src/main.rs", "fn main() {}")
+        .build();
+
+    assert_that(p.cargo("publish").masquerade_as_nightly_cargo()
+                 .arg("--registry").arg("alternative").arg("-Zunstable-options"),
+                execs().with_status(101).with_stderr("\
+[ERROR] some crates cannot be published.
+`foo` is marked as unpublishable
+"));
+}
+
+#[test]
+fn publish_allowed_registry() {
+    publish::setup();
+
+    let p = project("foo").build();
+
+    let _ = repo(&paths::root().join("foo"))
+        .file("Cargo.toml", r#"
+            cargo-features = ["alternative-registries"]
+
+            [project]
+            name = "foo"
+            version = "0.0.1"
+            authors = []
+            license = "MIT"
+            description = "foo"
+            documentation = "foo"
+            homepage = "foo"
+            publish = ["alternative"]
+        "#)
+        .file("src/main.rs", "fn main() {}")
+        .build();
+
+    assert_that(p.cargo("publish").masquerade_as_nightly_cargo()
+                 .arg("--registry").arg("alternative").arg("-Zunstable-options"),
+                execs().with_status(0));
+}
+
+#[test]
+fn block_publish_no_registry() {
+    publish::setup();
+
+    let p = project("foo")
+        .file("Cargo.toml", r#"
+            cargo-features = ["alternative-registries"]
+
+            [project]
+            name = "foo"
+            version = "0.0.1"
+            authors = []
+            license = "MIT"
+            description = "foo"
+            publish = []
+        "#)
+        .file("src/main.rs", "fn main() {}")
+        .build();
+
+    assert_that(p.cargo("publish").masquerade_as_nightly_cargo()
+                 .arg("--index").arg(publish::registry().to_string()),
+                execs().with_status(101).with_stderr("\
+[ERROR] some crates cannot be published.
+`foo` is marked as unpublishable
+"));
 }
